@@ -3,6 +3,20 @@
 
 namespace internal {
 
+template <typename ElementType>
+struct DataRescaler
+{
+    float slope, intercept;
+    
+    DataRescaler (const float slope, const float intercept)
+        : slope(slope), intercept(intercept) {}
+    
+    inline ElementType operator() (const ElementType value)
+    {
+        return static_cast<ElementType>(value * slope + intercept);
+    }
+};
+
 template <typename SourceType, typename TargetType>
 inline TargetType convertValue (SourceType value)
 {
@@ -186,6 +200,8 @@ inline short stringToDatatype (const std::string &datatype)
         return datatypeCodes[lowerCaseDatatype];
 }
 
+#ifndef _NO_R__
+
 template <typename TargetType>
 inline void copyIfPresent (const Rcpp::List &list, const std::set<std::string> names, const std::string &name, TargetType &target)
 {
@@ -201,6 +217,8 @@ inline void copyIfPresent (const Rcpp::List &list, const std::set<std::string> n
         target = static_cast<char>(Rcpp::as<int>(list[name]));
 }
 
+#endif // _NO_R__
+
 inline mat33 topLeftCorner (const mat44 &matrix)
 {
     mat33 newMatrix;
@@ -213,6 +231,8 @@ inline mat33 topLeftCorner (const mat44 &matrix)
     return newMatrix;
 }
 
+#ifndef _NO_R__
+
 template <int SexpType>
 inline Rcpp::RObject imageDataToArray (const nifti_image *source)
 {
@@ -277,6 +297,8 @@ inline void addAttributes (Rcpp::RObject &object, nifti_image *source, const boo
     object.attr(".nifti_image_ptr") = xptr;
 }
 
+#endif // _NO_R__
+
 } // namespace
 
 #endif
